/*!
 * rivet-core - @version 2.0.0-alpha.2

 * Copyright (C) 2018 The Trustees of Indiana University
 * SPDX-License-Identifier: BSD-3-Clause
 */

if (!Element.prototype.closest) {
  if (!Element.prototype.matches) {
    Element.prototype.matches =
      Element.prototype.msMatchesSelector ||
      Element.prototype.webkitMatchesSelector;
  }
  Element.prototype.closest = function(s) {
    var el = this;
    var ancestor = this;
    if (!document.documentElement.contains(el)) return null;
    do {
      if (ancestor.matches(s)) return ancestor;
      ancestor = ancestor.parentElement;
    } while (ancestor !== null);
    return null;
  };
}


(function() {
  if (typeof window.CustomEvent === 'function') return false;

  function CustomEvent(event, params) {
    params = params || { bubbles: false, cancelable: false, detail: undefined };
    var evt = document.createEvent('CustomEvent');
    evt.initCustomEvent(
      event,
      params.bubbles,
      params.cancelable,
      params.detail
    );
    return evt;
  }

  CustomEvent.prototype = window.Event.prototype;

  window.CustomEvent = CustomEvent;
})();

(function(arr) {
  arr.forEach(function(item) {
    if (item.hasOwnProperty('remove')) {
      return;
    }
    Object.defineProperty(item, 'remove', {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function remove() {
        if (this.parentNode === null) {
          return;
        }
        this.parentNode.removeChild(this);
      }
    });
  });
})([Element.prototype, CharacterData.prototype, DocumentType.prototype]);

const globalSettings = {
  prefix: 'rvt'
};

const { prefix } = globalSettings;

function dispatchCustomEvent(eventName, element, detail) {
  const newEvent = new CustomEvent(`${prefix}:${eventName}`, {
    bubbles: true,
    cancelable: true,
    detail
  });

  const eventShouldFire = element.dispatchEvent(newEvent);
  return eventShouldFire;
}


const nodeListToArray = nodes => Array.prototype.slice.call(nodes);

const isNode = element =>
  element.nodeType === 1 || element.nodeType === 1;


const keyCodes = {
  up: 38,
  down: 40,
  left: 37,
  right: 39,
  tab: 9,
  enter: 13,
  escape: 27,
  home: 36,
  end: 35,
  pageUp: 33,
  pageDown: 34
};


class Accordion {
  constructor(element, options) {
    const defaultOptions = {
      openAllOnInit: false
    };

    const settings = {
      ...defaultOptions,
      ...options
    };

    this.openAllOnInit = settings.openAllOnInit;
    this.element = element;

    if (!isNode(this.element)) {
      throw new TypeError(
        'A DOM element should be passed as the first argument to initialize the accordion.'
      );
    }

    this.triggerAttribute = 'data-accordion-trigger';
    this.triggerSelector = `[${this.triggerAttribute}]`;
    this.triggers = nodeListToArray(
      this.element.querySelectorAll(this.triggerSelector)
    );
    if (this.triggers.length < 2) {
      throw new TypeError(
        'An accordion should contain *at least two* accordion triggers with the "data-accordion-trigger" attribute'
      );
    }
    this.panelAttribute = 'data-accordion-panel';
    this.panelSelector = `[${this.panelAttribute}]`;
    this.panels = nodeListToArray(
      this.element.querySelectorAll(this.panelSelector)
    );
    const initializedPanels = this.element.querySelectorAll(
      `[data-accordion-panel-init]`
    );

    try {
      if (initializedPanels.length > 1) {
        throw new TypeError('Caught');
      }

      let initialPanel;

      this.panels.forEach((panel, index) => {
        if (panel.hasAttribute('data-accordion-panel-init')) {
          initialPanel = panel;
        } else {
          this.panels[index].setAttribute('hidden', '');
        }
      });

      this.openOnInit = initialPanel;
    } catch (e) {
      console.warn(
        'Only one accordion panel should have the data-accordion-panel-init attribute. If you wish to open all panels on initialization, please apply the appropriate attribute to the data-accordion element'
      );
    }

    this._handleClick = this._handleClick.bind(this);
    this._handleKeydown = this._handleKeydown.bind(this);

    this.init();
  }

  _handleClick(event) {
    const currentTrigger = event.target.closest(this.triggerSelector);
    if (!currentTrigger) return;

    const currentTriggerValue = currentTrigger.getAttribute(
      this.triggerAttribute
    );

    const currentPanel = this.element.querySelector(
      `[${this.panelAttribute}="${currentTriggerValue}"]`
    );

    currentTrigger.getAttribute('aria-expanded') === 'true'
      ? this.close(currentPanel)
      : this.open(currentPanel);
  }

  _handleKeydown(event) {
    if (
      event.keyCode === keyCodes.up ||
      event.keyCode === keyCodes.down ||
      event.keyCode === keyCodes.end ||
      event.keyCode === keyCodes.home
    ) {
      const accordionParent = event.target.closest('[data-accordion]');
      if (!accordionParent) return;

      const currentTrigger = event.target.closest(this.triggerSelector);
      if (!currentTrigger) return;

      const nextTrigger = this.triggers.indexOf(currentTrigger) + 1;
      const prevTrigger = this.triggers.indexOf(currentTrigger) - 1;

      switch (event.keyCode) {
        case keyCodes.up: {
          if (this.triggers[prevTrigger] === undefined) {
            this.triggers[this.triggers.length - 1].focus();
          } else {
            this.triggers[prevTrigger].focus();
          }
          break;
        }

        case keyCodes.down: {
          if (this.triggers[nextTrigger] === undefined) {
            this.triggers[0].focus();
          } else {
            this.triggers[nextTrigger].focus();
          }
          break;
        }

        case keyCodes.end: {
          this.triggers[this.triggers.length - 1].focus();
          break;
        }

        case keyCodes.home: {
          this.triggers[0].focus();
          break;
        }
      }
    }
  }

  _openOnInit() {
    if (this.openAllOnInit === true) {
      this.panels.forEach(panel => {
        this.open(panel);
      });
    } else {
      this.panels.forEach((panel, index) => {
        if (panel !== this.openOnInit) {
          this.panels[index].setAttribute('hidden', '');
          const trigger = this.element.querySelector(
            `[${this.triggerAttribute}="${panel.dataset.accordionPanel}"]`
          );
          trigger.setAttribute('aria-expanded', 'false');
        } else {
          this.open(this.openOnInit);
        }
      });
    }
  }

  open(accordion, callback) {

    const activationEvent = dispatchCustomEvent('accordionOpened', accordion, {
      id: accordion.dataset.accordionPanel
    });

    if (!activationEvent) return;

    const trigger = this.element.querySelector(
      `[${this.triggerAttribute} = "${accordion.dataset.accordionPanel}"]`
    );

    accordion.removeAttribute('hidden');
    trigger.setAttribute('aria-expanded', 'true');

    if (callback && typeof callback === 'function') {
      callback();
    }
  }

  close(accordion, callback) {

    const activationEvent = dispatchCustomEvent('accordionClosed', accordion, {
      id: accordion.dataset.accordionPanel
    });

    if (!activationEvent) return;

    const trigger = this.element.querySelector(
      `[${this.triggerAttribute} = "${accordion.dataset.accordionPanel}"]`
    );

    accordion.setAttribute('hidden', '');
    trigger.setAttribute('aria-expanded', 'false');

    if (callback && typeof callback === 'function') {
      callback();
    }
  }

  init() {
    this._openOnInit();

    this.element.addEventListener('click', this._handleClick, false);
    this.element.addEventListener('keydown', this._handleKeydown, false);
  }

  destroy() {
    this.element.removeEventListener('click', this._handleClick, false);
    this.element.removeEventListener('keydown', this._handleKeydown, false);
  }
}


class Alert {
  constructor(element) {
    this.element = element;
    this.closeAttribute = 'data-alert-close';
    this.closeSelector = `[${this.closeAttribute}]`;
    this.closeButton = null;

    this._handleClick = this._handleClick.bind(this);

    if (!isNode(this.element)) {
      throw new TypeError(
        'A DOM element should be passed as the first argument to initialize the alert'
      );
    }

    this.init();
  }

  _handleClick() {
    this.dismiss();
  }

  dismiss() {
    const dismissEvent = dispatchCustomEvent('alertDismiss', this.element, {
      id: this.element.dataset.alert
    });

    if (!dismissEvent) return;

    this.destroy();
    this.element.remove();
  }

  init() {
    this.closeButton = this.element.querySelector(this.closeSelector);
    this.closeButton.addEventListener('click', this._handleClick, false);
  }

  destroy() {
    this.closeButton.addEventListener('click', this._handleClick, false);
  }
}


class Dropdown {
  constructor(element) {
    this.element = element;
    this.focusableElements =
      'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="0"]';

    this.dropdownAttribute = '[data-dropdown]';

    this.toggleAttribute = '[data-dropdown-toggle]';
    this.toggleElement = this.element.querySelector(this.toggleAttribute);

    this.menuAttribute = '[data-dropdown-menu]';
    this.menuElement = this.element.querySelector(this.menuAttribute);

    this.isOpen = false;
    this.activeToggle = null;
    this.activeMenu = null;

    this._handleClick = this._handleClick.bind(this);
    this._handleKeydown = this._handleKeydown.bind(this);

    this.element.querySelector('svg').setAttribute('focusable', 'false');

    this.init();
  }

  open() {
    if (this.toggleElement.hasAttribute('disabled')) {
      return;
    }

    const openEvent = dispatchCustomEvent('dropdownOpen', this.toggleElement, {
      id: this.toggleElement.dataset.dropdownToggle
    });

    if (!openEvent) return;

    this.isOpen = true;

    this.toggleElement.setAttribute('aria-expanded', 'true');

    this.menuElement.removeAttribute('hidden');

    this.activeToggle = this.toggleElement;
    this.activeMenu = this.menuElement;
  }

  close() {
    if (!this.activeToggle) return;

    const closeEvent = dispatchCustomEvent(
      'dropdownClose',
      this.toggleElement,
      {
        id: this.toggleElement.dataset.dropdownToggle
      }
    );

    if (!closeEvent) return;

    this.isOpen = false;

    this.activeToggle.setAttribute('aria-expanded', 'false');
    this.activeMenu.setAttribute('hidden', '');

    this.activeToggle = null;
    this.activeMenu = null;
  }

  _setUpMenu(menu) {
    const menuObject = {};

    const menuFocusables = nodeListToArray(
      menu.querySelectorAll(this.focusableElements)
    );

    menuObject.all = menuFocusables;

    menuObject.first = menuFocusables[0];

    menuObject.last = menuFocusables[menuFocusables.length - 1];

    return menuObject;
  }

  _handleClick(event) {
    const toggle = event.target.closest(this.toggleAttribute);

    if (this.menuElement.contains(event.target)) return;

    if (!toggle && this.activeToggle !== null) {
      this.close();
      return;
    }

    if (toggle !== this.toggleElement || this.isOpen) {
      this.close();
    } else {
      this.open();
    }
  }

  _handleKeydown(event) {
    if (!this.element.contains(event.target)) return;

    const dropdown = event.target.closest(this.dropdownAttribute);
    if (dropdown !== this.element) return;

    switch (event.keyCode) {
      case keyCodes.down: {
        event.preventDefault();

        if (!this.isOpen) {
          this.open();
        } else {
          const currentMenu = this._setUpMenu(this.menuElement);
          currentMenu.first.focus();
        }

        if (!this.menuElement.contains(event.target)) break;

        const currentMenu = this._setUpMenu(this.menuElement);
        let currentIndex;

        for (let i = 0; i < currentMenu.all.length; i++) {
          if (event.target == currentMenu.all[i]) {
            currentIndex = i;
          }
        }

        const nextItem = currentMenu.all[currentIndex + 1];

        if (!nextItem) {
          currentMenu.first.focus();

          return;
        }

        nextItem.focus();

        break;
      }

      case keyCodes.up: {
        event.preventDefault();

        if (!this.menuElement.contains(event.target)) break;

        const currentMenu = this._setUpMenu(this.menuElement);
        let currentIndex;

        for (let i = 0; i < currentMenu.all.length; i++) {
          if (event.target == currentMenu.all[i]) {
            currentIndex = i;
          }
        }

        const previousItem = currentMenu.all[currentIndex - 1];

        if (!previousItem && currentMenu.last !== undefined) {
          currentMenu.last.focus();

          return;
        }

        previousItem.focus();

        break;
      }

      case keyCodes.escape: {
        if (!this.activeToggle) return;

        this.close();

        this.toggleElement.focus();

        this.activeToggle = null;

        break;
      }

      case keyCodes.tab: {
        if (!this.menuElement.contains(event.target)) break;

        const currentMenu = this._setUpMenu(this.menuElement);

        if (document.activeElement == currentMenu.last && !event.shiftKey) {
          this.close();

          return;
        }

        break;
      }
    }
  }

  init() {
    document.addEventListener('click', this._handleClick, false);
    document.addEventListener('keydown', this._handleKeydown, false);
  }

  destroy() {
    document.removeEventListener('click', this._handleClick, false);
    document.removeEventListener('keydown', this._handleKeydown, false);
  }
}


class FileInput {
  constructor(element, options) {
    const defaultOptions = {
      previewText: 'No file selected'
    };

    const settings = {
      ...defaultOptions,
      ...options
    };

    this.element = element;
    this.wrapperAttribute = 'data-upload-wrapper';
    this.inputAttribute = 'data-upload-input';
    this.previewAttribute = 'data-upload-preview';
    this.previewText = settings.previewText;

    this._handleChange = this._handleChange.bind(this);

    this.init();
  }

  _sanitizeHTML(str) {
    const temp = document.createElement('div');
    temp.textContent = str;
    return temp.innerHTML;
  }

  _buildSingleFile(input) {
    const singleFileItem = document.createElement('span');

    const singleFileName = this._sanitizeHTML(input.files[0].name);

    singleFileItem.textContent = singleFileName;

    return singleFileItem;
  }

  _buildMultipleFiles(input) {
    const fileCount = document.createElement('span');
    fileCount.textContent = input.files.length + ' files selected';

    return fileCount;
  }

  _handleChange() {
    const uploadInput = this.element.querySelector(`[${this.inputAttribute}]`);

    const uploadPreview = this.element.querySelector(
      `[${this.previewAttribute}]`
    );

    if (uploadInput.files.length > 0) {
      uploadPreview.innerHTML = '';

      uploadInput.files.length > 1
        ? uploadPreview.appendChild(this._buildMultipleFiles(uploadInput))
        : uploadPreview.appendChild(this._buildSingleFile(uploadInput));
    } else {
      uploadPreview.innerHTML = this.previewText;
    }
  }

  init() {
    this.element.addEventListener('change', this._handleChange, false);
  }

  destroy() {
    this.element.removeEventListener('change', this._handleChange, false);
  }
}


class Modal {
  constructor(element, options) {
    const defaultOptions = {
      openOnInit: false,
      dialog: false
    };

    const settings = {
      ...defaultOptions,
      ...options
    };

    this.element = element;

    if (!isNode(this.element)) {
      throw new TypeError(
        'A DOM element should be passed as the first argument to initialize the modal.'
      );
    }

    this.openOnInit = settings.openOnInit;
    this.dialog = settings.dialog;
    this.modalAttribute = 'data-modal';
    this.modalSelector = `[${this.modalAttribute}]`;
    this.modalDataValue = this.element.getAttribute('data-modal');
    this.innerModalAttribute = 'data-modal-inner';
    this.innerModalSelector = `[${this.innerModalAttribute}]`;
    this.openAttribute = 'data-modal-trigger';
    this.openSelector = `[${this.openAttribute}]`;
    this.openButton = document.querySelector(
      `[${this.openAttribute}="${this.modalDataValue}"]`
    );
    this.closeAttribute = 'data-modal-close';
    this.closeSelector = `[${this.closeAttribute}]`;

    this.focusElements =
      'a[href], area[href], input:not([disabled]), select:not([disabled]), textarea:not([disabled]), button:not([disabled]), [tabindex="-1"]';

    this._handleClick = this._handleClick.bind(this);
    this._handleKeydown = this._handleKeydown.bind(this);

    this.init();
  }

  _handleClick(event) {
    event.target.closest(this.innerModalSelector) !== null
      ? (event.clickedInModal = true)
      : (event.clickedInModal = false);

    if (event.clickedInModal) {
      event.stopPropagation();
    }

    const triggerSelectors = `${this.openSelector}, ${this.closeSelector}, ${this.modalSelector}`;
    const trigger = event.target.closest(triggerSelectors);
    if (!trigger) return;

    let triggerContent;

    if (trigger.getAttribute(this.openAttribute)) {
      triggerContent = trigger.getAttribute(this.openAttribute);
    } else if (trigger.getAttribute(this.closeAttribute)) {
      triggerContent = trigger.getAttribute(this.closeAttribute);
    } else {
      triggerContent = event.target.closest(this.modalSelector);
    }

    if (!triggerContent) return;

    switch (trigger != null) {
      case trigger.hasAttribute(this.openAttribute): {
        if (trigger.getAttribute(this.openAttribute) !== this.modalDataValue) {
          this.close();
          return;
        }

        this.open();

        this.element.focus();

        break;
      }
      case trigger.hasAttribute(this.closeAttribute): {
        event.preventDefault();

        if (trigger.getAttribute(this.closeAttribute) !== this.modalDataValue)
          return;
        this.close();

        if (this.openButton !== null) this.openButton.focus();

        break;
      }
      case !event.clickedInModal: {
        if (this.dialog) return;

        if (
          triggerContent.getAttribute(this.modalAttribute) !==
          this.modalDataValue
        )
          return;
        this.close();

        if (this.openButton !== null) this.openButton.focus();

        break;
      }
      default: {
        return;
      }
    }
  }

  _handleBackwardTab(first, last, event) {
    if (
      document.activeElement === first ||
      document.activeElement === this.element
    ) {
      event.preventDefault();
      last.focus();
    }
  }

  _handleForwardTab(first, last, event) {
    if (document.activeElement === last) {
      event.preventDefault();
      first.focus();
    }
  }

  _handleKeydown(event) {
    const currentModal = event.target.closest(this.modalSelector);
    if (!currentModal) return;

    switch (event.keyCode) {
      case keyCodes.tab: {
        const focusList = nodeListToArray(
          currentModal.querySelectorAll(this.focusElements)
        );
        const firstFocus = focusList[0];
        const finalFocus = focusList[focusList.length - 1];

        event.shiftKey
          ? this._handleBackwardTab(firstFocus, finalFocus, event)
          : this._handleForwardTab(firstFocus, finalFocus, event);

        break;
      }
      case keyCodes.escape: {
        if (this.dialog) return;

        if (
          currentModal.getAttribute(this.modalAttribute) !== this.modalDataValue
        )
          return;

        this.close();

        if (this.openButton !== null) this.openButton.focus();

        break;
      }
    }
  }

  open(callback) {
    const openEvent = dispatchCustomEvent('modalOpen', this.element, {
      id: this.element.dataset.modal
    });

    if (!openEvent) return;

    this.element.removeAttribute('hidden');
    document.body.classList.add('rvt-modal-open');

    if (callback && typeof callback === 'function') {
      callback();
    }
  }

  close(callback) {
    const closeEvent = dispatchCustomEvent('modalClose', this.element, {
      id: this.element.dataset.modal
    });

    if (!closeEvent) return;

    this.element.setAttribute('hidden', '');
    document.body.classList.remove('rvt-modal-open');

    if (callback && typeof callback === 'function') {
      callback();
    }
  }

  focusTrigger() {
    if (!this.openButton) {
      throw new Error(
        `Could not find a modal trigger with the value of ${this.modalDataValue}`
      );
    }

    this.openButton.focus();
  }

  focusModal() {
    if (!this.element) {
      throw new Error(
        `Could not find a modal with the value of ${this.modalDataValue}`
      );
    }

    this.element.focus();
  }

  init() {
    if (!this.openOnInit) {
      this.element.setAttribute('hidden', '');
    }

    this.destroy();

    document.addEventListener('click', this._handleClick, false);
    document.addEventListener('keydown', this._handleKeydown, false);
  }

  destroy() {
    document.removeEventListener('click', this._handleClick, false);
    document.removeEventListener('keydown', this._handleKeydown, false);
  }
}


class Sidenav {
  constructor(element, options) {
    const defaultOptions = {
      openAllOnInit: false
    };

    const settings = {
      ...defaultOptions,
      ...options
    };

    this.element = element;
    this.openAllOnInit = settings.openAllOnInit;
    this.toggleAttribute = 'data-sidenav-toggle';
    this.toggleSelector = `[${this.toggleAttribute}]`;
    this.listAttribute = 'data-sidenav-list';
    this.listSelector = `[${this.listAttribute}]`;

    this._handleClick = this._handleClick.bind(this);

    if (!isNode(this.element)) {
      throw new TypeError(
        'A DOM element should be passed as the first argument to initialize the sidenav'
      );
    }

    this.init();
  }

  _handleClick(event) {
    const toggleButton = event.target.closest(this.toggleSelector);
    if (!toggleButton) return;

    const toggleId = toggleButton.dataset.sidenavToggle;
    const targetList = this.element.querySelector(
      `[${this.listAttribute}="${toggleId}"]`
    );

    if (!targetList || targetList.getAttribute(this.listAttribute) === '') {
      return;
    }

    targetList.hasAttribute('hidden')
      ? this.open(toggleButton, targetList)
      : this.close(toggleButton, targetList);
  }

  open(toggleButton, targetList) {
    const openEvent = dispatchCustomEvent('sidenavListOpen', toggleButton, {
      id: toggleButton.dataset.sidenavToggle
    });

    if (!openEvent) return;

    toggleButton.setAttribute('aria-expanded', 'true');
    targetList.removeAttribute('hidden');
  }

  close(toggleButton, targetList) {
    const closeEvent = dispatchCustomEvent('sidenavListClose', toggleButton, {
      id: toggleButton.dataset.sidenavToggle
    });

    if (!closeEvent) return;

    toggleButton.setAttribute('aria-expanded', 'false');
    targetList.setAttribute('hidden', '');
  }

  init() {
    this.element.addEventListener('click', this._handleClick, false);

    const menuToggles = nodeListToArray(
      this.element.querySelectorAll(this.toggleSelector)
    );
    const childMenus = nodeListToArray(
      this.element.querySelectorAll(this.listSelector)
    );

    menuToggles.forEach((element, index) => {

      element.setAttribute('aria-haspopup', 'true');

      if (this.openAllOnInit) {
        element.setAttribute('aria-expanded', 'true');
        childMenus[index].removeAttribute('hidden');

        return;
      }

      if (element.getAttribute('aria-expanded') === 'true') {
        childMenus[index].removeAttribute('hidden');
      } else {
        element.setAttribute('aria-expanded', 'false');
        childMenus[index].setAttribute('hidden', '');
      }
    });
  }

  destroy() {
    this.element.removeEventListener('click', this._handleClick, false);
  }
}


class Tabs {
  constructor(element) {
    this.element = element;

    if (!isNode(this.element)) {
      throw new TypeError(
        'A DOM element should be passed as the first argument to initialize the modal.'
      );
    }

    this.tablist = this.element.querySelector('[role="tablist"]');
    this.tabAttribute = 'data-tab';
    this.tabSelector = `[${this.tabAttribute}]`;
    this.tabs = nodeListToArray(
      this.element.querySelectorAll(this.tabSelector)
    );
    this.panelAttribute = 'data-tab-panel';
    this.panelSelector = `[${this.panelAttribute}]`;
    this.panels = nodeListToArray(
      this.element.querySelectorAll(this.panelSelector)
    );

    let initialPanel;
    this.panels.forEach((panel, index) => {
      if (panel.hasAttribute('data-tab-init')) {
        initialPanel = panel;
      } else {
        this.tabs[index].setAttribute('tabindex', '-1');
      }
    });

    this.openOnInit = initialPanel || this.panels[0];

    this._handleClick = this._handleClick.bind(this);
    this._handleKeydown = this._handleKeydown.bind(this);

    this.init();
  }

  _handleClick(event) {
    const currentTab = event.target.closest(this.tabSelector);
    if (!currentTab) return;

    const currentTabValue = currentTab.getAttribute(this.tabAttribute);

    const currentPanel = this.element.querySelector(
      `[${this.panelAttribute}="${currentTabValue}"]`
    );

    this.activateTab(currentPanel);
  }

  _handleKeydown(event) {
    const currentTab = event.target.closest(this.tabSelector);
    if (!currentTab) return;

    const nextTab = this.tabs.indexOf(currentTab) + 1;
    const prevTab = this.tabs.indexOf(currentTab) - 1;

    switch (event.keyCode) {
      case keyCodes.right : {
        !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
        break;
      }

      case keyCodes.down: {
        !this.tabs[nextTab] ? this.tabs[0].focus() : this.tabs[nextTab].focus();
        break;
      }

      case keyCodes.left: {
        !this.tabs[prevTab]
          ? this.tabs[this.tabs.length - 1].focus()
          : this.tabs[prevTab].focus();
        break;
      }

      case keyCodes.up: {
        !this.tabs[prevTab]
          ? this.tabs[this.tabs.length - 1].focus()
          : this.tabs[prevTab].focus();
        break;
      }

      case keyCodes.end: {
        this.tabs[this.tabs.length - 1].focus();
        break;
      }

      case keyCodes.home: {
        this.tabs[0].focus();
        break;
      }
    }
  }

  activateTab(tab, callback) {

    const activationEvent = dispatchCustomEvent('tabActivated', tab, {
      id: tab.dataset.tabPanel
    });

    if (!activationEvent) return;

    const trigger = this.element.querySelector(
      `[${this.tabAttribute}="${tab.dataset.tabPanel}"]`
    );

    this.panels.forEach((panel, index) => {
      if (panel.getAttribute(this.panelAttribute) !== tab.dataset.tabPanel) {
        panel.setAttribute('hidden', '');
        this.tabs[index].setAttribute('aria-selected', 'false');
        this.tabs[index].setAttribute('tabindex', '-1');
      }
    });

    tab.removeAttribute('hidden');
    trigger.setAttribute('aria-selected', 'true');
    trigger.removeAttribute('tabindex');

    if (callback && typeof callback === 'function') {
      callback();
    }
  }

  init() {
    this.activateTab(this.openOnInit);
    this.destroy();

    this.tablist.addEventListener('click', this._handleClick, false);
    this.tablist.addEventListener('keydown', this._handleKeydown, false);
  }

  destroy() {
    this.tablist.removeEventListener('click', this._handleClick, false);
    this.tablist.removeEventListener('keydown', this._handleKeydown, false);
  }
}

export { Accordion, Alert, Dropdown, FileInput, Modal, Sidenav, Tabs };
